/**
 * Author: Andreas Gasper
 */

/**
 * statements_in_method(+MethodID, -Statements)
 *
 * Succeeds iff the Statements are part of the method with MethodID
 */


    
statements_in_method(MethodID, Statements):-
	ri_blockT_encl(MethodID, BlockID),
	blockT(BlockID, _, MethodID, Statements)
	.

statements_in_method(MethodID, Statements):-
    ri_switchT_encl(MethodID, SwitchID),
    switchT(SwitchID, _, MethodID, _, Statements)
    .

statements_in_method(MethodID, [ThenStatement]):-
    ri_ifT_encl(MethodID, IfID),
    ifT(IfID, _, MethodID, _, ThenStatement, _),
    \+(blockT(ThenStatement, _, _, _))
    .
    
statements_in_method(MethodID, [ElseStatement]):-
    ri_ifT_encl(MethodID, IfID),
    ifT(IfID, _, MethodID, _, _, ElseStatement),
    \+(blockT(ElseStatement, _, _, _)),
    \+(ElseStatement == 'null')
    . 

statements_in_method(MethodID, [WhileStatement]):-
	ri_whileT_encl(MethodID, WhileID),
	whileT(WhileID, _, MethodID, _, WhileStatement),
	\+(blockT(WhileStatement, _, _, _))
    .

statements_in_method(MethodID, [WhileStatement]):-
	ri_doWhileT_encl(MethodID, WhileID),
	doWhileT(WhileID, _, MethodID, _, WhileStatement),
	\+(blockT(WhileStatement, _, _, _))
    . 
    
statements_in_method(MethodID, [ForStatement]):-
	ri_forT_encl(MethodID, ForID),
	forT(ForID, _, MethodID, _, _, _, ForStatement),
	\+(blockT(ForStatement, _, _, _))
    . 

statements_in_method(MethodID,[ForeachStatement]):-
	ri_foreachT_encl(MethodID, ForEachID),
	foreachT(ForEachID, _, MethodID, _, _, ForeachStatement),
	\+(blockT(ForeachStatement, _, _, _))
    .   

statements_in_method(MethodID,[CatchID]):-
    ri_catchT_encl(MethodID, CatchID)
    .
    
/************************************************************************************/
%statements_in_method(MethodID, Statements):-
%	blockT(_, _, MethodID, Statements)
%	.
%
%statements_in_method(MethodID, Statements):-
%    switchT(_, _, MethodID, _, Statements)
%    .
%
%statements_in_method(MethodID, [ThenStatement]):-
%    ifT(_, _, MethodID, _, ThenStatement, _),
%    \+(blockT(ThenStatement, _, _, _))
%    .
%    
%statements_in_method(MethodID, [ElseStatement]):-
%    ifT(_, _, MethodID, _, _, ElseStatement),
%    \+(blockT(ElseStatement, _, _, _)),
%    \+(ElseStatement == 'null')
%    . 
%
%statements_in_method(MethodID, [WhileStatement]):-
%	whileT(_, _, MethodID, _, WhileStatement),
%	\+(blockT(WhileStatement, _, _, _))
%    .
%
%statements_in_method(MethodID, [WhileStatement]):-
%	doWhileT(_, _, MethodID, _, WhileStatement),
%	\+(blockT(WhileStatement, _, _, _))
%    . 
%    
%statements_in_method(MethodID, [ForStatement]):-
%	forT(_, _, MethodID, _, _, _, ForStatement),
%	\+(blockT(ForStatement, _, _, _))
%    . 
%
%statements_in_method(MethodID,[ForeachStatement]):-
%	foreachT(_, _, MethodID, _, _, ForeachStatement),
%	\+(blockT(ForeachStatement, _, _, _))
%    . 

/************************************************************************************/    
% Not usable since its the most inefficent way (JHotDraw 28,05)...
 
%statements_in_method_i(MethodID, Statements):-
%	findall(StatementID,
%				(
%					statement(StatementID, _, MethodID)
%				),
%			Statements
%	)
%	. 

%testing(L):-
%    findall(Statements, (methodDef(MethodID, _,_,_,_,_,_),statements_in_method(MethodID,Statements)),L). 

%asdf(D):-
%    findall(Statements, (methodDef(MethodID, _,_,_,_,_,_),statements_in_method(MethodID,Statements)),L),
%    findall(IStatements, (methodDef(IMethodID, _,_,_,_,_,_),statements_in_method_i(IMethodID,IStatements)),IL),
%    flatten(L,LF),
%    flatten(IL,ILF),
%    intersection(ILF,LF,C),
%    subtract(ILF,C,D).
/**
 * Kommentare von dsp:
 *
 * Tut vermutlich, was du von ihm erwartest. Ist noch ineffizient. 
 * 
 * - Auf jeden Fall solltest du die ri_ Fakten oder ein darauf aufbauendes Pr�dikat verwenden. So wie du 
 *   hier vorgehst, l�ufst du jedesmal ein drittel der Faktenbasis ab.
 * AG: Auf ri_ umgestellt, bei JHotDraw von 14,52 auf 0,06 Sekunden, also Faktor 242, verbessert...
 *
 * - Bei allen Verwendungen (bis auf das Z�hlen der Statements) dieses Pr�dikats holst du als n�chstes
 *   jedes einzelne Statement aus dem Array. Daher macht wohl (auch/eher) ein Pr�dikat method_contains_statement
 *   (Singular!) Sinn. 
 * - Wenn du f�r die Z�hlerei diese Version des Pr�dikats behalten m�chtest, dann benenn es z.B. in 
 *   method_contains_statement_sequence oder method_contains_statements um.
 * - Ich find "not()" viel lesbarer als "\+()", aber das ist nur ganz pers�nlicher Geschmack.
 * AG: Ich mag \+() lieber :-) 
 * - Vielleicht macht es Sinn, die vielen F�lle noch durch ein zus�tzliches Pr�dikat zu reduzieren. 
 * - Du k�nntest nat�rlich direkt alle in der Methode enthaltenen Statements erfassen, ohne die Blocks zu ber�cksichtigen.
 *   Vermutlich ist die Verwendung der Blocks aber besser.
 * AG: Die Blocks nehme ich weil es dadurch weniger Pr�dikate sind die man schreiben muss als wenn man f�r jedes einzelne Statement das Pr�dikat schreiben muss.
 * - Man k�nnte dar�ber nachdenken, die Initialisierung und das Weiterz�hlen in der for-Schleife auch als Statements
 *   zu z�hlen. (L�nge des Arrays eher als nur einfach.) Allerdings w�re es dann vermutlich schwierig, diese Sicht
 *   konsequent durchzuhalten. Beispiel: int i, j; for(i=1, j=2; (i+j) <9; i++, j++). Was meinst du?
 * AG: Ich finde eigentlich dass die nicht als Statements z�hlen sollten. 
 *     Das i ist ja nur im Kontext der for-Schleife sichtbar und geh�rt f�r mich mehr zum for dazu als dass es ein eigenes Statement
 *     w�re. Das Beispiel m�sste dann auch so aussehen {int i, j; for(i=1, j=2; (i+j) <9; i++, j++)}.
 
 */