filtered(Type, Name, Paths, Participant) :- 
    is_list(Paths),
    member(Path, Paths),
	filtered(Type, Name, Path, Participant).
	    
filtered(smell, Name, Path, Participant) :-
    not(is_list(Path)),
    file_filtering(Path, Name, Participant),
    smell(Name, Participant)
    .
    
filtered(relation_smell, Name, Path, FirstParticipant, SecondParticipant) :-
    not(is_list(Path)),
    file_filtering(Path, Name, FirstParticipant),
    relation_smell(Name, FirstParticipant, SecondParticipant),
    !.
    
filtered(relation_smell, Name, Path, FirstParticipant, SecondParticipant) :-
    not(is_list(Path)),
    file_filtering(Path, Name, SecondParticipant),
    relation_smell(Name, FirstParticipant, SecondParticipant)
    .    
    
filtered(metric, Name, Path, Participant, MetricValue, RangeAndThreshold) :-
    not(is_list(Path)),
    file_filtering(Path, Name, Participant),
    metric(Name, Participant, MetricValue, RangeAndThreshold)
    .
    
filtered(relation_metric, Name, Path, FirstParticipant, SecondParticipant, MetricValue, RangeAndThreshold) :-
    not(is_list(Path)),
    file_filtering(Path, Name, FirstParticipant),
    relation_metric(Name, FirstParticipant, SecondParticipant, MetricValue, RangeAndThreshold),
    !.
    
filtered(relation_metric, Name, Path, FirstParticipant, SecondParticipant, MetricValue, RangeAndThreshold) :-
    not(is_list(Path)),
    file_filtering(Path, Name, SecondParticipant),
    relation_metric(Name, FirstParticipant, SecondParticipant, MetricValue, RangeAndThreshold)
    .
    
/* file filtering: explicit scope*/    
file_filtering(Path, Name, Participant) :-    
    member(QueryType, [smell, metric, relation_smell, relation_metric]),
    query_scope(Name, QueryType, Scope), !,
    scoped_file_filtering(Path, Scope, Participant).
    
/* file filtering: routes to legacy
*/    
file_filtering(Path, _Name, Participant) :-    
    file_filtering(Path, Participant).
        
/* file filtering: scope inference
 * 
 * file filtering with scope inference from first goal in smell/metric definition
 * 
 * possible goals are: 
 *  named_internal_type, 
 *  internal_type, 
 *  internal_package,
 *  internal_method
 */
file_filtering(Path, Name, Participant) :-
	not(query_scope(Name, QueryType, Scope)),    
    member(QueryType, [smell, metric, relation_smell, relation_metric]),
    infer_scope(Name, QueryType, Scope), !,
    scoped_file_filtering(Path, Scope, Participant).
    
infer_scope(Name, metric, Scope) :-
	clause(metric_(Name, _Participant, _Value), Body),
	Body =.. ListOfGoals,
	nth1(2, ListOfGoals, FirstConstraint),
	scope_for_constraint(FirstConstraint, Scope)
	.
	
infer_scope(Name, smell, Scope) :-
	clause(smell_(Name, _Participant), Body),
	Body =.. ListOfGoals,
	nth1(2, ListOfGoals, FirstConstraint),
	scope_for_constraint(FirstConstraint, Scope)
	.		
	
scope_for_constraint(internal_package(_P), packageT) :-
	!.
		
scope_for_constraint(named_internal_type(_P), classT) :-
	!.
	
scope_for_constraint(internal_type(_P), classT) :-
	!.
	
scope_for_constraint(internal_method(_P), methodDefT) :-
	!.	
	    

/* 
*/    
scoped_file_filtering(Path, compilationUnitT, CU) :-
    fileS(File, _, Path), !,
    ri_compilationUnitT_file(File, CU).
    
scoped_file_filtering(Path, classT, Type) :-
	scoped_file_filtering(Path, compilationUnitT, CU), !, 
	compilationunit_contains_type(CU, Type).
	
scoped_file_filtering(Path, methodDef, Method) :-
	scoped_file_filtering(Path, classT, Type), !,
	type_contains_method(Type, Method).

scoped_file_filtering(Path, fieldT, Field) :-
	scoped_file_filtering(Path, classT, Type), !,
	type_contains_field(Type, Field).

/* generic scoped
* 
* scoped_file_filtering(+Path, +Scope, ?Context)
*/
scoped_file_filtering(Path, Scope, Context) :-
    not(member(Scope, [compilationUnitT, classT, methodDef, fieldT])),

    fileS(File, _, Path), 
    ri_compilationUnitT_file(File, CU),
    compilationunit_contains_type(CU, Type),
    type_contains_method(Type, Method),
    
    enclosing(Context, methodDef, Method),
    ast_node_type_value(Context, Scope) 
	.	
			
/* legacy
*/
file_filtering(Path, CU) :-
    scoped_file_filtering(Path, compilationUnitT, CU).
file_filtering(Path, Type) :-
	scoped_file_filtering(Path, classT, Type).
file_filtering(Path, Method) :-	
	scoped_file_filtering(Path, methodDef, Method).
file_filtering(Path, Field) :-	
	scoped_file_filtering(Path, fieldT, Field).
		
