
/**
 * find_source_location(+PEF,-Filename,-Offset,-Length)
 *   
 * Arg1 is the id of a class.
 * Arg2 is the absolute path of Arg1 relative to the workspace.
 * Arg3 is the starting point in file if the Arg1 is a class ID. 
 * Arg4 is the length of the source portion representing the Arg1.  
 * 
 * @JIRA CV-257
 */

find_source_location(PEF,Filename,Offset,Length):-
    reduced_source_location(PEF, Offset, Length),
    encl_path_down(PEF,[_,_,ToplevelClass|_]),
    fileS(FileId, _, Filename),
    compilationUnitT(_, _, FileId, _, Types),
    memberchk(ToplevelClass,Types). 
    
 /**
 * path_of_package(Package,Path)
 *
 * Arg2 is the path of the package with ID Arg1.
 * The path is relative to the current workspace.
 * The predicate assumes that all compilation units of a package
 * reside in the same folder. THIS IS NOT STANDARD JAVA!!!
 */
find_source_location(Package,FolderPath, Offset, Length):-
    compilationUnitT(_,Package, File, _,_),
    fileS(File, _, FilePath),
    file_directory_name(FilePath, FolderPath),
    Offset = 0,
    Length = 0.
/**
* reduced_source_location(+PEF, -Offset, -Length) 
*
*/    
reduced_source_location(PEF, Offset, Length) :-
    methodDef(PEF, _, _, _, _, _, Block), 
    blockT(Block, _, _, _),
    slT(PEF, Offset, _),
    slT(Block, OffsetBlock, _),
    Length is OffsetBlock - Offset, 
    !.

reduced_source_location(PEF, Offset, Length) :-
    classT(PEF, _, _, [FirstMember| _]), 
    slT(PEF, Offset, _),
    slT(FirstMember, OffsetFirst, _),
    Length is OffsetFirst - Offset, 
    !.

reduced_source_location(PEF, Offset, Length) :-
    classT(PEF, _, _, [_HiddenDefaultConstructor, SecondMember | _]), 
    slT(PEF, Offset, _),
    slT(SecondMember, OffsetFirst, _),
    Length is OffsetFirst - Offset, 
    !.
    
reduced_source_location(PEF, Offset, Length) :-
    slT(PEF,Offset,Length).
    
  