/** 
    dependency_term(-DependentId, -DependeeId) :-  
		A package depends on another package if a term is introduced (for introduction definition see term_is_introduced/2 in term_cache.pl)
		in the dependee package and appears in the regular term cloud of the dependent package.
		
        If there are no dependencies based on terms, detector returns false.
        Otherwise it returns the two involved package ids and the term the dependency is based on.       
*/

term_precondition__ :-
   once(no_cache_available) -> fill_term_cache ; true.
   
dependency_termbased(DependentId,DependeeId) :-
    term_precondition__,
    termbased_dependency__(DependentId, DependeeId).

/** 
    dependency_term(+DependentId, +DependeeId, -DependentTerms) :-  
   succeeds iff  DependentTerms is a list of Terms which Arg1 introduces and Arg2 is dependent on.     
*/
depend_terms_for_packages(DependentId,DependeeId, DependentTerms):-
    get_all_dependend_terms_for_packages__(DependentId, DependeeId, DependentTerms).  

/** 
    degree_of_depend_terms(+DependentId, +DependeeId, -Degree) :-  
   succeeds iff  Degree is the length of the List of Terms which Arg1 introduces and Arg2 is dependent on.     
*/

:- multifile query_description/3.
:- multifile relation_metric_/4.
:- multifile metric_attributes/2.
query_description(termDependencyDegree, relation_metric, 'dependency degree between two packages based on terms').
metric_attributes(termDependencyDegree, int(range(0, infinity), defaultthresholds(0, infinity))).

/*
* termDependencyDegree(+DependentPackage, +DependeePackage, ?DepenencyDegree)
* 
* Succeeds if Arg3 is the Degree between Arg1 and Arg2 based on terms.
*/     
relation_metric_(termDependencyDegree, DependentId, DependeeId, DependencyDegree) :-
     termbased_dependency__(DependentId, DependeeId),
     depend_terms_for_packages(DependentId,DependeeId, DegreeList),
     length(DegreeList,DependencyDegree).
         
     
	
%
%   termbased_dependency__(?DependentID, ?DependeeID)
%
%   Returns true if there is a dependency between DependentID Package and DependeeID Package, i.e.
%   if there is a term introduced in DependeeID Package and is a member of all terms of DependentID Package
%
%   TODO: Add results to cache.	
termbased_dependency__(DependentId, DependeeId) :-
	internal_package(DependentId), 
	internal_package(DependeeId),
	not(DependentId == DependeeId),
	once((package_contains_type(DependeeId, DependeeClassId), 
	      is_term_introduced(DependeeClassId, Term,_),
	      package_contains_type(DependentId, DependentClassId), 
	      term(context_has_term(DependentClassId,Term,_))
	    )).
	
	
/**
 get_all_dependend_terms_for_packages__/3
  +DependentId
  +DependeeId
  ?ListOfContributingTerms
 
 retrieves a list of terms contributing to a term dependency relation between two packages
*/
get_all_dependend_terms_for_packages__(DependentId, DependeeId, SortedListOfContributingTerms):-
    ground([DependentId,DependeeId]),
    findall(Term, 
    			(   package_contains_type(DependeeId, ClassId),
    				is_term_introduced(ClassId, Term,true), 
    				package_contains_type(DependentId, DependentClassId),
    				not(DependeeId == DependentId),
    				term(context_has_term(DependentClassId, Term,_)))
    		, ListOfContributingTerms),
     not(length(ListOfContributingTerms,0)),		
     sort(ListOfContributingTerms,SortedListOfContributingTerms).	


query_description(consistencyViolation, relation_smell, 'consistency violation').
 
relation_smell_(consistencyViolation, DependentId, DependeeId):-
    		dependency_termbased(DependentId,DependeeId),
    		not(relation_metric(packageDependencyDegree, DependentId, DependeeId, _)).
    		