
:- module( term_metrics, [
            clear_term_metrics/0,
            fill_term_metrics/0,

            cached_term_metrics_tf/3, %(Context,Term,MetricValue)
            cached_term_metrics_idf/3, %(Context,Term,MetricValue)
            cached_term_metrics_tfidf/3 %(Context,Term,MetricValue)
            
   ] ).

:- dynamic  cached_term_metrics_tf/3.
:- dynamic  cached_term_metrics_idf/3.
:- dynamic  cached_term_metrics_tfidf/3.


:- dynamic  cached_term_surprise/4. % (ContextLevel,Term,NumberDocumentsContainingTerm)

:- use_module('../term_vocabulary/term_model').
:- use_module('../term_vocabulary/bag_of_words_computation').
% Map to old one:
clear_term_metrics :- clear_term_metrics__.
fill_term_metrics :- fill_term_metrics__.

% New Retract
clear_term_metrics__  :-
    retractall( cached_term_metrics_tf(_,_,_) ),
    retractall( cached_term_metrics_idf(_,_,_) ),
    retractall( cached_term_metrics_tfidf(_,_,_) ),
    retractall( cached_term_surprise(_,_,_,_)).
% term_metrics

term_precondition__ :-
   once(no_cache_available) -> fill_term_cache ; true.
   
metric_precondition:-
   once(no_metric_cache_available) -> fill_term_metrics ; true.



no_metric_cache_available :- 
   not(cached_term_metrics_tf(_,_,_)).
           
fill_term_metrics__:-
   term_precondition__,
   generate_termfrequency__,
   generate_termsurprise_packagescope__,
   generate_termsurprise_classscope__,
   generate_termsurprise_methodscope__,
   generate_term_tfidf__.
  


% compute_termfrequency_with_classscope
% +ClassContext
% +Term
% ?TermFrequency  = number of occurences for this term in the class / number of all term occurences in the class   
%
generate_termfrequency__:-
    cached_occurrences_of_term_in_context(Context, Term, TermCount),
    cached_occurrences_of_all_terms_in_context(Context,TotalTermCountInDocument),
    TF is TermCount/TotalTermCountInDocument,
    assert( cached_term_metrics_tf(Context,Term,TF)),
    fail.
generate_termfrequency__.


/**
*
*
**/
termsurprise_documents__(Level,Context,Term,Value):-
    cached_term_surprise(Level,Context,Term,Value),!.

termsurprise_documents__(package,_,Term,Value):-
    findall(Package, (internal_package(Package),  cached_occurrences_of_term_in_context(Package,Term,_)), InternalPackagesContainingTerm),
    sort(InternalPackagesContainingTerm,SortedInternalPackagesContainingTerm),
    length(SortedInternalPackagesContainingTerm,Value),
   	   	assert( cached_term_surprise(package,'',Term,Value)).
   

termsurprise_documents__(type,Package,Term,Value):-
    findall(Class, (package_contains_type(Package,Class),  cached_occurrences_of_term_in_context(Class,Term,_)),ClassIdsContainingTerm),
    sort(ClassIdsContainingTerm,SortClassIdsContainingTerm),
    length(SortClassIdsContainingTerm,Value),
   	   	assert( cached_term_surprise(type,Package,Term,Value)).
   	   	
termsurprise_documents__(method,Type,Term,Value):-
    findall(MethodT, (type_contains_method(Type,MethodT),  cached_occurrences_of_term_in_context(MethodT,Term,_)), MethodsContainingTermList),
    sort(MethodsContainingTermList,SortedMethodsContainingTermList),
    length(SortedMethodsContainingTermList,Value),
   	   	assert( cached_term_surprise(method,Type,Term,Value)).   	   	

/**
*
*
**/
generate_termsurprise(type,Term,Value):-
    findall(Package, (internal_package(Package),  cached_occurrences_of_term_in_context(Package,Term,_)), InternalPackagesContainingTerm),
    sort(InternalPackagesContainingTerm,SortedInternalPackagesContainingTerm),
    length(SortedInternalPackagesContainingTerm,Value),
   	   	assert( cached_term_surprise(package,Term,Value)).

    
generate_termsurprise_packagescope__:-
    findall(Package, (internal_package(Package)), InternalPackages),
    length(InternalPackages,NrOfInternalPackages),
    internal_package(Context),
    cached_occurrences_of_term_in_context(Context,Term,_),
    termsurprise_documents__(package,_,Term,NrPackagesContainingTerm),
    
    TempSurprise is NrOfInternalPackages/NrPackagesContainingTerm,
    Surprise is (log(TempSurprise)/log(2)),
    	assert( cached_term_metrics_idf(Context,Term,Surprise)),
    fail. 
generate_termsurprise_packagescope__.    
     
     
generate_termsurprise_classscope__:-
    internal_package(Package),
    metric(numberOfTypes, Package, NumberOfTypes),
    package_contains_type(Package,Context),
    cached_occurrences_of_term_in_context(Context,Term,_), 
    
    termsurprise_documents__(type,Package,Term,NrOfClassesContainingTermInPackage),
       
    TempSurprise is NumberOfTypes/NrOfClassesContainingTermInPackage,
    Surprise is (log(TempSurprise)/log(2)),
    	assert( cached_term_metrics_idf(Context,Term,Surprise)),
    fail.
 generate_termsurprise_classscope__.

generate_termsurprise_methodscope__:-
    findall(Method, (internal_method(Method)), MethodList),
    length(MethodList,NrOfMethodsInClass),
    
    internal_package(Package),
    package_contains_type(Package,Type),
    
    type_contains_method(Type,Context),
    cached_occurrences_of_term_in_context(Context,Term,_),
    
    termsurprise_documents__(method,Type,Term,NrMethodsContainingTerm),
     
    TempSurprise is NrOfMethodsInClass/NrMethodsContainingTerm,
    Surprise is (log(TempSurprise)/log(2)),
    	assert( cached_term_metrics_idf(Context,Term,Surprise)),
    fail. 
generate_termsurprise_methodscope__.    




generate_term_tfidf__:-
    relevant_context_element(Context),
    cached_term_metrics_tf(Context,Term,TF),
      cached_term_metrics_idf(Context,Term,IDF),
      TFxIDF is TF*IDF,
      assert( cached_term_metrics_tfidf(Context,Term,TFxIDF)),
    fail.
 generate_term_tfidf__.    
/**
 FOR DEBUGGING ONLY!
*/
debugbreakpoint:- true.    