:- module('cultivate_concepts_tagging',[ fill_tmp_file/0]).

/**
 * internal_package(?Package)
 *
 * Suceeds if all the types in the package are internal.
 * But the empty packages is not internal. 
 */
%internal_package(Package) :-
%    packageT(Package, _),
%    once(package_contains_type(Package, Type)),
%    forall(package_contains_type(Package, Type), not(externT(Type))).

    /**
*  package_contains_type(+Package, ?Type)
*  succeeds iff package with ID Arg1 contains a type with ID Arg2
*/
%package_contains_type(Package, Type) :-
%   ri_compilationUnitT_parent(Package, CompilationUnit),
%   ri_classT_parent(CompilationUnit, Type).
:- use_module(library(process)).
:- use_module('../term_vocabulary/term_model').

fill_tmp_file:-
    	  clear__,
    	  fill_tmp_file(File,_),
		  test(File,_),
		  !.
clear__:-
    	retractall(tmp:count(_)),
    	retractall(tmp:current_id(_)),
    	retractall(tagging:pos(_,_)),
    	retractall(tagging:lemma(_,_)).
    	
fill_tmp_file(File,Stream):-
    			tmp_file_stream(text,File,Stream),
    			forall( (lexical_compound(ID,TermList), not(length(TermList,0))),
    				( format(Stream,'~w~n',[ID]),
    				forall(member(Term,TermList), format(Stream,'~w~n',[Term])),
    				format(Stream,'.~n',[]),
    				flush_output(Stream))),
    			close(Stream).	
			    			

:- if(current_prolog_flag(windows,true) ).
  treetagger_bin('tree-tagger.exe').
  treetagger_path('win/').
:- endif.

:- if(current_prolog_flag(unix,true) ).
  treetagger_bin('tree-tagger').
  treetagger_path('linux/').
:- endif.


tree_tagger_cmd(Command,ParameterFile):-
    		predicate_property(cultivate_concept_location_fact, file(File)),
    		file_directory_name(File,ThisProjectPath),
    		treetagger_bin(Binary),
  			treetagger_path(OS_PATH),
	     	atom_concat(RootPath, '/prolog', ThisProjectPath),
	     	atom_list_concat([RootPath,'/tagger/',OS_PATH,Binary],Command),
	     	atom_list_concat([RootPath,'/tagger/lib/english.par'],ParameterFile),!.
	     	
test(File,Stream):-
    			tmp_file_stream(text,_,Stream),
    			tree_tagger_cmd(Command,ParameterFile),
  				process_create(Command, 
	  					['-no-unknown', '-sgml','-token', '-lemma',ParameterFile,File], 
	  					[stdin(std),stdout(pipe(Out)),window(false),process(Process)]),
  				repeat,
  					read_line_to_codes(Out, Codes),
  					process(Codes,Stream),
  				!,	
  				delete_file(File),
  				close(Stream).	
  		

process('end_of_file',_):-!.
process(Codes,Stream) :-
        atom_codes(Atom,Codes),
		atomic_list_concat([A,B,C],'\t',Atom),
		format(Stream,'~w ~w ~w~n',[A,B,C]),
	%	format('~w ~w ~w~n',[A,B,C]),
		parse_pos_result(A,B,C),
		flush_output(Stream), 
	fail.

% + + + 	 
parse_pos_result(Word,'CD','@card@'):-
    atom_number(Word,ID),
	lexical_compound(ID,_),
	assert(tmp:current_id(ID)), 
	assert(tmp:count(1)),
	!.

parse_pos_result('.','SENT','.'):- 
	retract(tmp:current_id(_)),
	retract(tmp:count(_)),
	!.

parse_pos_result(Word,POS,Lemma):-
	tmp:current_id(Id),
	tmp:count(Position),
	termid(Id,Position,TermId),
	lexical_term(TermId,Word),
	assert(tagging:pos(TermId,POS)),
	assert(tagging:lemma(TermId,Lemma)),
	retract(tmp:count(Position)),
	NewPosition is Position + 1,
	assert(tmp:count(NewPosition)).
	
	%%
	% 10095 CD @card@
	% java NN java
	% lang JJ lang
 	% . SENT .
	%%
	%
	%		 
	 % tmp_file(+Base, -TmpName)
     % ID
     % W1
     % W2
     % W3
     % . 