:- multifile query_description/3.
:- multifile metric_/3.
:- multifile metric_attributes/2.

query_description(efferentCouplings,metric, 	'number of classes inside this package that depend on classes outside this package').
metric_attributes(efferentCouplings, int(range(0,infinity), defaultthresholds(0,infinity))).

          
/** *****************************************************************
  * Metric:  efferentCouplings
  * Return:  ?Integer
  * Context: ?PackageId, ?PackageName
  *
  * Returns the number of classes / interfaces inside the package
  * PackageId/PackageName that depend on clases / interfaces outside
  * the package.
  * The class depended on does not need to be imported explicitly.
  * That means that dependencies on java.lang will be counted as
  * well as dependencies to other classes that can be used witout
  * being imported (TODO: Explain the latter case / give example).
  *
  * For external packages the predicate fails.
  */
metric_(efferentCouplings,PackageID,EfferentCouplings) :-
    internal_package(PackageID),
    findall(ClassID,
            efferentDependencies(PackageID, ClassID),
            List),
    sort(List,DuplicateFree),
    length(DuplicateFree,EfferentCouplings).


/**
  * efferentDependencies(+Owner,?ClassID)
  *
  *  Succeeds if a class / interface inside the Package 'Owner'
  *  depends on the non-external(!!!) class / interface 'ClassID'.
  */
efferentDependencies(Package,ReferencedClassID) :-
    ri_compilationUnitT_parent(Package, CompilationUnit),
    ri_classT_parent(CompilationUnit, ClassInPackageID),
    packageT(OtherPackage,_name),
    not(OtherPackage = Package),
    ri_compilationUnitT_parent(OtherPackage, OtherCompilationUnit),
    ri_classT_parent(OtherCompilationUnit, ReferencedClassID),
    not(externT(ReferencedClassID)), %%% <-- Do we want this??? - GK
   	classDependency(ClassInPackageID, ReferencedClassID, _).
   	
   	